@@ -31,6 +31,10 @@
 #include "unicode-block-canvas.h"
 #include "buffered-write-sequencer.h"
 
+#ifdef WITH_TIMG_TERMINAL_QUERY
+#  include "term-query.h"
+#endif
+
 // To display version number
 #ifdef WITH_TIMG_OPENSLIDE_SUPPORT
 #  include "openslide-source.h"
@@ -150,7 +154,10 @@ static int usage(const char *progname, ExitCode exit_code,
     fprintf(stderr, "usage: %s [options] <%s> [<%s>...]\n", progname,
             kFileType, kFileType);
     fprintf(stderr, "\e[1mOptions\e[0m:\n"
-            "\t-g<w>x<h>      : Output geometry in character cells. Terminal is %dx%d\n"
+            "\t-g<w>x<h>      : Output geometry in character cells. Partial geometry\n"
+            "\t                 leaving out one value -g<w>x or -gx<h> is possible,\n"
+            "\t                 the other value it then derived from the terminal size.\n"
+            "\t                 Default derived from terminal size is %dx%d\n"
             "\t-p<pixelation> : Pixelation: 'h' = half blocks    'q' = quarter blocks\n"
             "\t                             'k' = kitty graphics 'i' = iTerm2 graphics\n"
             "\t                 Default: Auto-detect graphics, otherwise 'quarter'.\n"
@@ -181,7 +188,12 @@ static int usage(const char *progname, ExitCode exit_code,
             "\t                 decoding first (useful, if you stream video from stdin)\n"
             "\t-I             : Only  use Image subsystem. Don't attempt video decoding\n"
 #endif
-            "\t-F, --title    : Print filename as title above each image.\n"
+            "\t--title[=<fmt_str>]: Print title above each image. Accepts the following\n"
+            "\t                 format parameters: %%f = full filename; %%b = basename\n"
+            "\t                                    %%w = image width; %%h = height\n"
+            "\t                                    %%D = internal decoder used\n"
+            "\t                 If no parameter is given, defaults to \"%%f\"\n"
+            "\t-F             : Print filename as title. Behaves like --title=\"%%f\"\n"
             "\t-f<filelist>   : Read newline-separated list of image files to show.\n"
             "\t                 (Can be provided multiple times.)\n"
             "\t-o<outfile>    : Write to <outfile> instead of stdout.\n"
@@ -288,9 +300,10 @@ static void PresentImages(LoadedImageSources &loaded_sources,
         std::unique_ptr<timg::ImageSource> source(source_future.get());
         if (!source) continue;
         before_image_show(is_first);
-        source->SendFrames(present.duration_per_image,
-                           present.loops, interrupt_received,
-                           renderer->render_cb(source->filename().c_str()));
+        source->SendFrames(present.duration_per_image, present.loops,
+                           interrupt_received,
+                           renderer->render_cb(
+                               source->FormatTitle(display_opts.title_format)));
         after_image_show();
         if (!present.duration_between_images.is_zero()) {
             (Time::Now() + present.duration_between_images).WaitUntil();
@@ -369,7 +382,7 @@ int main(int argc, char *argv[]) {
         { "rotate",      required_argument, NULL, OPT_ROTATE },
         { "scroll",      optional_argument, NULL, 's' },
         { "threads",     required_argument, NULL, OPT_THREADS },
-        { "title",       no_argument,       NULL, 'F' },
+        { "title",       optional_argument, NULL, 'F' },
         { "upscale",     optional_argument, NULL, 'U' },
         { "verbose",     no_argument,       NULL, OPT_VERBOSE },
         { "version",     no_argument,       NULL, OPT_VERSION },
@@ -389,8 +402,10 @@ int main(int argc, char *argv[]) {
                               long_options, &option_index))!=-1) {
         switch (opt) {
         case 'g':
-            if (sscanf(optarg, "%dx%d",
-                       &geometry_width, &geometry_height) < 2) {
+            // Parse xHEIGHT, WIDTHx, WIDTHxHEIGHT
+            if ((sscanf(optarg, "x%d", &geometry_height) == 0) &&
+                (sscanf(optarg, "%dx%d",
+                        &geometry_width, &geometry_height) < 1)) {
                 fprintf(stderr, "Invalid size spec '%s'", optarg);
                 return usage(argv[0], ExitCode::kParameterError,
                              geometry_width, geometry_height);
@@ -517,7 +532,8 @@ int main(int argc, char *argv[]) {
             }
             break;
         case 'F':
-            display_opts.show_filename = !display_opts.show_filename;
+            display_opts.show_title = !display_opts.show_title;
+            if (optarg) display_opts.title_format = optarg;
             break;
         case 'E':
             present.hide_cursor = false;
@@ -628,6 +644,7 @@ int main(int argc, char *argv[]) {
     // Determine best default to pixelate images.
     if (present.pixelation == Pixelation::kNotChosen) {
         present.pixelation = Pixelation::kQuarterBlock;  // Good default.
+#ifdef WITH_TIMG_TERMINAL_QUERY
         // Konsole has the bad behaviour that it does not absorb the kitty
         // graphics query but spills it on the screen. "Luckily", Konsole has
         // another bug not returning the window pixel size, so we can use that
@@ -638,6 +655,7 @@ int main(int argc, char *argv[]) {
             else if (timg::QueryHasKittyGraphics())
                 present.pixelation = Pixelation::kKittyGraphics;
         }
+#endif
     }
 
     // If 'none' is chosen for background color, then using the
@@ -721,7 +739,7 @@ int main(int argc, char *argv[]) {
         present.loops = 1;  // Don't get stuck on the first endless-loop
     }
 
-    if (display_opts.show_filename) {
+    if (display_opts.show_title) {
         // Leave space for text.
         display_opts.height -= display_opts.cell_y_px * present.grid_rows;
     }
@@ -734,14 +752,19 @@ int main(int argc, char *argv[]) {
     std::future<rgba_t> background_color_future;
     if (bg_color) {
         if (strcasecmp(bg_color, "auto") == 0) {
+#ifdef WITH_TIMG_TERMINAL_QUERY
             std::function<rgba_t()> query_terminal = []() {
-                return rgba_t::ParseColor(timg::DetermineBackgroundColor());
+                return rgba_t::ParseColor(timg::QueryBackgroundColor());
             };
             background_color_future = pool->ExecAsync(query_terminal);
             display_opts.bgcolor_getter = [&background_color_future]() {
                 static rgba_t value = background_color_future.get(); // once
                 return value;
             };
+#else
+            const rgba_t bg = rgba_t::ParseColor("#000000");
+            display_opts.bgcolor_getter = [bg]() { return bg; };
+#endif
         }
         else {
             const rgba_t bg = rgba_t::ParseColor(bg_color);
@@ -757,18 +780,22 @@ int main(int argc, char *argv[]) {
 
     ExitCode exit_code = ExitCode::kSuccess;
 
+    // Only print errors if we got one explicit filename.
+    const bool print_errors = (filelist.size() == 1);
+
     // Async image loading, preparing them in a thread pool
     LoadedImageSources loaded_sources;
     for (const std::string &filename : filelist) {
         if (interrupt_received) break;
         std::function<timg::ImageSource*()> f =
-            [filename, frame_offset, max_frames, do_img_loading, do_vid_loading,
+            [filename, frame_offset, max_frames,
+             do_img_loading, do_vid_loading, print_errors,
              &display_opts, &exit_code]() -> timg::ImageSource* {
                 if (interrupt_received) return nullptr;
                 auto result = ImageSource::Create(filename, display_opts,
                                                   frame_offset, max_frames,
                                                   do_img_loading,
-                                                  do_vid_loading);
+                                                  do_vid_loading, print_errors);
                 if (!result) exit_code = ExitCode::kImageReadError;
                 return result;
             };
@@ -801,6 +828,8 @@ int main(int argc, char *argv[]) {
     if (verbose) {
         fprintf(stderr, "Terminal cells: %dx%d  cell-pixels: %dx%d\n",
                 term.cols, term.rows, term.font_width_px, term.font_height_px);
+        fprintf(stderr, "Active Geometry: %dx%d\n",
+                geometry_width, geometry_height);
         const Duration d = end_show - start_show;
         const uint64_t written_bytes =
             sequencer.bytes_total() - sequencer.bytes_skipped();
